
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>


enum {
	// _xinfreq = 20_000_000,
	_xtlfreq = 20_000_000,
	_clkfreq = 100_000_000,
	DOWNLOAD_BAUD = 230_400,
	DEBUG_BAUD = DOWNLOAD_BAUD,

};



uint32_t  pinwait( int Temp_pin, int settling, uint32_t mode ) __attribute__((cog))
{
    uint32_t  sample, diff1 = 0, diff2 = 0;

    _rdpin( Temp_pin );
    _wrpin( Temp_pin, mode );

    do {
        while( !_pinr( Temp_pin ) );    // wait for next decimation
        sample = _rdpin( Temp_pin ) << 5;    // 27-bit to 32-bit translation for clean rotary summing
        sample -= diff1;
        diff1 += sample;
        sample -= diff2;
        diff2 += sample;
    } while( --settling );

    return sample >> 10; // sinc3 x 7 (128 period) + 5 (translation) - 10 = 16-bit samples
}

/*
uint32_t  pinwait( int temp_pin, int settling, uint32_t mode ) __attribute__((cog))
{
    uint32_t  sample, diff1 = 0, diff2 = 0;

	__asm volatile { // no optimising, ensures instruction order
		rdpin	sample, temp_pin
		wrpin	mode, temp_pin
.loop1
		testp	temp_pin   wc
	if_nc	jmp	#.loop1

		rdpin	sample, temp_pin
		shl	sample, #5
		sub	sample, diff1
		add	diff1, sample
		sub	sample, diff2
		add	diff2, sample
		djnz	settling, #.loop1
	}

    return sample >> 10; // sinc3 x 7 (128 period) + 5 (translation) - 10 = 16-bit samples
}
*/

enum {
	PINBIAS = P_ADC,
//	PINBIAS = P_ADC | P_OE | P_LOW_150K,
//	PINBIAS = P_ADC | P_OE | P_HIGH_150K | P_INVERT_OUTPUT,
};


void  tsr_loop( int Temp_pin )
{
    uint32_t  value, vio, gio, timer;
    float  temperature, ratio;

    _pinstart( Temp_pin, P_ADC_1X | PINBIAS, 0b10_0000, 128 ); // "Sinc3 Filtering" mode, 128 ticks per decimation, rated 14 ENOB
    _waitms( 10 );    // warm up the ADC hardware circuits
    printf( "      pin P%d\n", Temp_pin );

    for( int i = 3; i<=19; i++ ) {
        timer = _cnt();
        vio = pinwait( Temp_pin, 4, P_ADC_VIO | PINBIAS );
        gio = pinwait( Temp_pin, 4, P_ADC_GIO | PINBIAS );
        value = pinwait( Temp_pin, i, P_ADC_1X | PINBIAS );    // must not be first in order
        timer = _cnt() - timer;
        printf( "duration = %d  ", timer );

        ratio = (float)(vio - gio) / (float)(value - gio) - 1.0;    // Rt connected to GIO, R0 connected to VIO
//        ratio = (float)(vio - gio) / (float)(vio - value) - 1.0;    // Rt connected to VIO, R0 connected to GIO
        temperature = 1.0 / (log(ratio) / 3380.0 + 1.0 / 298.15) - 273.15;    // Thermistor Beta for 10 kR at 25 C

        printf( "gio = %d  vio = %d  value = %d  Temperature = %f  Ratio = %f  rln = %f\n",
                gio, vio, value, temperature, ratio, log(ratio) );
        _waitms( 100 );
    }

    _pinclear( Temp_pin );    // turn off the ADC
}



void main(void)
{

    _waitms(200);
    printf( "\n   clkfreq = %d   clkmode = 0x%x\n", _clockfreq(), _clockmode() );

    for(int i = 0; i<62; i++)
        tsr_loop(i);
}
